/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2023 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "./server.h"

#include "aqhome/aqhome.h"

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/args.h>
#include <gwenhywfar/logger.h>
#include <gwenhywfar/db.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/cgui.h>
#include <gwenhywfar/text.h>

#ifdef HAVE_SIGNAL_H
# include <signal.h>
#endif

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <time.h>



/* ------------------------------------------------------------------------------------------------
 * defines
 * ------------------------------------------------------------------------------------------------
 */

#define I18N(msg) msg
#define I18S(msg) msg

#define CONNCHECK_INTERVAL_IN_SECS  10
#define PING_INTERVAL_IN_SECS      120
#define SAVE_INTERVAL_IN_SECS       60

#define FULL_DEBUG



/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */

static void _runService(AQH_OBJECT *aqh, AQH_EVENT_LOOP *eventLoop);
static int _diffInSeconds(time_t t1, time_t t0);

#ifdef HAVE_SIGNAL_H
static int _setSignalHandlers(void);
static int _setupSigAction(struct sigaction *sa, int sig);
static void _signalHandler(int s);
static struct sigaction saINT,saTERM, saHUP, saTSTP, saCONT, saPIPE;
#endif



/* ------------------------------------------------------------------------------------------------
 * static vars
 * ------------------------------------------------------------------------------------------------
 */

static int stopService=0;



/* ------------------------------------------------------------------------------------------------
 * implementations
 * ------------------------------------------------------------------------------------------------
 */

int main(int argc, char **argv)
{
  AQH_EVENT_LOOP *eventLoop;
  AQH_OBJECT *aqh;
  int rv;
  GWEN_GUI *gui;

  rv=GWEN_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init Gwen.\n");
    return 2;
  }

  GWEN_Logger_Open(0, "aqhome-mqttlog", 0, GWEN_LoggerType_Console, GWEN_LoggerFacility_User);
  //GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Warning);
  GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Notice);

  rv=_setSignalHandlers();
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return rv;
  }

  rv=AQH_Init();
  if (rv<0) {
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  gui=GWEN_Gui_CGui_new();
  GWEN_Gui_SetGui(gui);

  eventLoop=AQH_EventLoop_new();

  aqh=AQH_MqttLogServer_new(eventLoop);
  rv=AQH_MqttLogServer_Init(aqh, argc, argv);
  if (rv<0) {
    if (rv==GWEN_ERROR_CLOSE)
      return 1;
    DBG_INFO(NULL, "here (%d)", rv);
    return 2;
  }

  _runService(aqh, eventLoop);


  AQH_MqttLogServer_Fini(aqh);
  AQH_Object_free(aqh);
  AQH_EventLoop_free(eventLoop);

  GWEN_Gui_SetGui(NULL);
  GWEN_Gui_free(gui);

  return 0;
}



void _runService(AQH_OBJECT *aqh, AQH_EVENT_LOOP *eventLoop)
{
  time_t timeStart;
  int timeout;
  time_t timeLastConnCheck;
  time_t timeLastSave;
  time_t timeLastPingSend;
  int rv;

  timeout=AQH_MqttLogServer_GetTimeout(aqh);
  timeStart=time(NULL);
  timeLastConnCheck=time(NULL);
  timeLastSave=time(NULL);
  timeLastPingSend=time(NULL);

  while(!stopService) {
    time_t now;

    AQH_EventLoop_Run(eventLoop, 2000);
    AQH_MqttLogServer_HandleMqttMsgs(aqh);
    AQH_MqttLogServer_HandleBrokerMsgs(aqh);

    now=time(NULL);

    if (_diffInSeconds(now, timeLastConnCheck)>CONNCHECK_INTERVAL_IN_SECS) {
      DBG_INFO(NULL, "Check connections");
      AQH_MqttLogServer_CheckBrokerConnection(aqh);
      AQH_MqttLogServer_CheckMqttConnection(aqh);
      timeLastConnCheck=now;
    }

    if (_diffInSeconds(now, timeLastPingSend)>PING_INTERVAL_IN_SECS) {
      rv=AQH_MqttLogServer_SendPing(aqh);
      if (rv<0) {
        DBG_INFO(NULL, "Error sending PING");
      }
      timeLastPingSend=time(NULL);
    }

    if (_diffInSeconds(now, timeLastSave)>SAVE_INTERVAL_IN_SECS) {
      DBG_INFO(NULL, "Writing device files");
      rv=AQH_MqttLogServer_SaveRuntimeDeviceFiles(aqh);
      if (rv<0) {
        DBG_INFO(NULL, "Error writing device file");
      }
      timeLastSave=time(NULL);
    }

    if (timeout && (_diffInSeconds(now, timeStart)>timeout)) {
      DBG_INFO(NULL, "Timeout");
      break;
    }
  } /* while */
  DBG_ERROR(NULL, "Leaving server");

  rv=AQH_MqttLogServer_SaveRuntimeDeviceFiles(aqh);
  if (rv<0) {
    DBG_INFO(NULL, "Error writing runtime data");
  }
}



int _setSignalHandlers(void)
{
#ifdef HAVE_SIGNAL_H
  int rv;

  rv=_setupSigAction(&saINT, SIGINT);
  if (rv)
    return rv;

  rv=_setupSigAction(&saTERM, SIGTERM);
  if (rv)
    return rv;

  rv=_setupSigAction(&saHUP, SIGHUP);
  if (rv)
    return rv;

  rv=_setupSigAction(&saPIPE, SIGPIPE);
  if (rv)
    return rv;

#  ifdef SIGTSTP
  rv=_setupSigAction(&saTSTP, SIGTSTP);
  if (rv)
    return rv;
#  endif

#  ifdef SIGCONT
  rv=_setupSigAction(&saCONT, SIGCONT);
  if (rv)
    return rv;
#  endif
#endif
  return 0;
}



int _setupSigAction(struct sigaction *sa, int sig)
{
  sa->sa_handler=_signalHandler;
  sigemptyset(&sa->sa_mask);
  sa->sa_flags=0;
  if (sigaction(sig, sa, 0)) {
    DBG_ERROR(NULL, "Could not setup signal handler for signal %d", sig);
    return GWEN_ERROR_IO;
  }

  return 0;
}



void _signalHandler(int s)
{
  switch(s) {
  case SIGINT:
  case SIGTERM:
  case SIGHUP:
    DBG_WARN(0, "Received signal %d, stopping service in next loop.",s);
    stopService=1;
    break;
  case SIGPIPE:
    DBG_WARN(0, "Received PIPE signal");
    break;
  default:
    DBG_WARN(0, "Unknown signal %d",s);
    break;
  }
}



int _diffInSeconds(time_t t1, time_t t0)
{
  return t1-t0;
}

